/*
    Back Screen Example #1
    by Charles MacDonald
    WWW: http://cgfm2.emuviews.com

    Based on the bmp8bpp.zip and startup.zip packages written by
    Bart Trzynadlowski. (www.dynarec.com/~bart)

    Uses the SH-COFF toolchain from KPIT Infosystems. (www.kpit.com)
    Load and run at address $06004000.

    Press START to exit the demo and return to the cheat cartridge menu.
    Press B to toggle the BDCLMD bit of the TVMD register.

    Overview:

    The back screen is a graphics layer generated by the VDP2 which is shown
    behind all other layers. It can be used to define the entire screen's
    backdrop color as a single color, or it can define the backdrop color
    for each scanline (e.g. for 224, 240, 448, or 480 lines).

    The following registers control the back screen:

    0x25F800AC - BKTAU
    0x25F800AE - BKTAL

    Bits 2-0 of BKTAU and 15-0 of BKTAL specify a 19-bit address which is
    the start address of the back screen table in VRAM. This provides a
    1MB address space, but the Saturn VDP2 only has 512K of VRAM. In this
    case, the most significant bit (bit 2 of BKTAU) is ignored and can be
    set to any value with the same results.

    Bit 15 of BKTAU has the following meanings:

    0= Use entry #0 from the table to select the color of every line in
       the display. (single color backdrop)

    1= Use each table entry to select the color of each line in the display.
       (multi-color backdrop)

    The table consists of 224, 240, 448, or 480 word-sized entries, depending
    on the display height. Each entry is a 5:5:5 RGB color value, which
    allows the back screen to use up to 32768 colors total.

    For interlaced displays, the table stores the color value for line 0
    of the even frame first, then the value of line 0 for the odd frame.
    The data is stored in an interleaved fashion, so this repeats until the
    last two entries which are the values for lines 479 of the even frame
    and line 479 of the odd frame.

    If bit 8 (BDCLMD) of TVMD (0x25F80000) is cleared, then any portion of
    the display outside of the active display area (meaning the horizontal
    and vertical borders) are filled with a black color.

    If this bit is set, then the horizontal borders use the same color as
    their respective scanline in the active display area used, and the
    vertical borders use the same color as the *last* line in the active
    display area used.

    The latter case is most noticable when using 224 or 448-line resolutions
    which usually have an eight scanline border at the top and bottom of the
    displays, and is less noticable for 240 or 480-line resolutions which
    have much smaller borders.

    In any case, you can simply set bit 8 of TVMD to keep the borders black
    if you don't want to worry about this.

    Because the back screen is shown behind all other layers, you need to
    ensure that the transparency status of the other backgrounds is active
    and not disabled. The back screen shows through transparent pixels in
    subsequent layers.
*/

#include "shared.h"

int sinetab[];

void _main()
{
    int frame = 0;
    uint16 tvmd_value = 0x8000;
    uint16 *bkp;
    uint32 address;
    uint16 joy_old, joy_new, joy_delta;

    vdp_init();
    conio_init();
    pad_init();

    /* Print messages and cursor */
    _printf(2, 1, 0xF0, "Back Screen Example #1");
    _printf(2, 2, 0xF0, "by Charles MacDonald");
    _printf(2, 3, 0xF0, "WWW: http://cgfm2.emuviews.com");
    _printf(2, 5, 0xF0, "TVMD BDCLMD=0");

    /* Address of back screen table */
    address = 0x40000;

    /* Set address and per-line color mode */
    BKTAU = 0x8000 | ((address >> 17) & 7);
    BKTAL = 0x0000 | ((address >> 1) & 0xFFFF);

    /* Set pointer to top of back screen table */
    bkp = (uint16 *)(VDP2_VRAM + address);

    /* Screen on */
    TVMD = tvmd_value;

    for(;;)
    {
        int i;
        int bary[3];

        wait_vblank_out();
        wait_vblank_in();
        frame++;

        /* Print state of BDCLMD bit */
        _printhexd(14, 5, 0xF0, (tvmd_value >> 8) & 1);

        /* Clear table */
        for(i = 0; i < 240; i++)
            bkp[i] = 0;

        /* Calculate bar positions */
        bary[0] = sinetab[(frame + 0) & 0xFF];
        bary[1] = sinetab[(frame + 16) & 0xFF];
        bary[2] = sinetab[(frame + 32) & 0xFF];

        /* Draw bars */
        for(i = 0; i < 64; i++)
        {
            int j = (i & 0x20) ? 0x1f-(i & 0x1f) : i;
            bkp[bary[0]+i] += j;
            bkp[bary[1]+i] += j<<5;
            bkp[bary[2]+i] += j<<10;
        }

        /* Poll joypad */
        joy_old = joy_new;
        joy_new = pad_read(0);
        joy_delta = (joy_new ^ joy_old) & joy_new;

        /* Start - Quit program */
        if(joy_new & PAD_START)
            break;

        /* Button B - Toggle BDCLMD bit */
        if(joy_delta & PAD_B)
        {
            tvmd_value ^= 0x0100;
            TVMD = tvmd_value;
        }
    }

    vdp_shutdown();
    pad_shutdown();
}

int sinetab[] =
{
  88,  90,  92,  94,  96,  98,  100,  102,
  105,  107,  109,  111,  113,  115,  117,  119,
  121,  123,  125,  127,  129,  131,  132,  134,
  136,  138,  140,  141,  143,  145,  146,  148,
  149,  151,  152,  154,  155,  156,  158,  159,
  160,  161,  163,  164,  165,  166,  167,  167,
  168,  169,  170,  171,  171,  172,  172,  173,
  173,  174,  174,  174,  175,  175,  175,  175,
  175,  175,  175,  175,  175,  174,  174,  174,
  173,  173,  172,  172,  171,  171,  170,  169,
  168,  167,  167,  166,  165,  164,  163,  161,
  160,  159,  158,  156,  155,  154,  152,  151,
  149,  148,  146,  145,  143,  141,  140,  138,
  136,  134,  132,  131,  129,  127,  125,  123,
  121,  119,  117,  115,  113,  111,  109,  107,
  105,  102,  100,  98,  96,  94,  92,  90,
  88,  85,  83,  81,  79,  77,  75,  73,
  70,  68,  66,  64,  62,  60,  58,  56,
  54,  52,  50,  48,  46,  44,  43,  41,
  39,  37,  35,  34,  32,  30,  29,  27,
  26,  24,  23,  21,  20,  19,  17,  16,
  15,  14,  12,  11,  10,  9,  8,  8,
  7,  6,  5,  4,  4,  3,  3,  2,
  2,  1,  1,  1,  0,  0,  0,  0,
  0,  0,  0,  0,  0,  1,  1,  1,
  2,  2,  3,  3,  4,  4,  5,  6,
  7,  8,  8,  9,  10,  11,  12,  14,
  15,  16,  17,  19,  20,  21,  23,  24,
  26,  27,  29,  30,  32,  34,  35,  37,
  39,  41,  43,  44,  46,  48,  50,  52,
  54,  56,  58,  60,  62,  64,  66,  68,
  70,  73,  75,  77,  79,  81,  83,  85
};

